@@ -1,10 +1,10 @@
 /* contentdir.c - Implementation of UPnP ContentDirectory
  *
- * Copyright (C) 2005, 2006  Oskar Liljeblad
+ * Copyright (C) 2005, 2006, 2007  Oskar Liljeblad
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -351,7 +351,7 @@ add_result_entry(TMap *results, Entry *entry, const char *filter)
                 strbuf_append(result, "<res");
 
                 if (filter_includes_any(filter, "res", "@protocolInfo", "res@protocolInfo", NULL))
-                    strbuf_appendf(result, " protocolInfo=\"http-get:*:%s:*\"", detail->data.file.mime_type);
+                    strbuf_appendf(result, " protocolInfo=\"http-get:*:%s:%s\"", detail->data.file.mime_type, detail->data.file.ext_info);
 
 		if (S_ISREG(detail->data.file.mode)) { /*if (detail->data.file.size != 0)*/
 	            if (filter_includes_any(filter, "res", "@size", "res@size", NULL))
@@ -532,7 +532,8 @@ parse_sort_criteria(const char *str, const char **error)
 	    return NULL;
 	}
 	entry = xmalloc(sizeof(SortCriteriaEntry));
-	entry->property = xmemdup(str, end-str);
+	entry->property = xmemdup(str, end-str+1);
+	entry->property[end-str] = 0;
 	entry->ascending = ascending;
 	entry->next = NULL;
 	if (criteria->first == NULL)
