@@ -1,10 +1,10 @@
 /* metadata.c - Management of file metadata
  *
- * Copyright (C) 2005, 2006  Oskar Liljeblad
+ * Copyright (C) 2005, 2006, 2007  Oskar Liljeblad
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -40,7 +40,7 @@
 #include <iconv.h>		/* POSIX */
 #include <magic.h>		/* libmagic */
 #include <upnp/ithread.h>	/* libupnp */
-#include "iconvme.h"		/* Gnulib */
+#include "striconv.h"		/* Gnulib */
 #include "gettext.h"		/* Gnulib/gettext */
 #define _(s) gettext(s)
 #define N_(s) gettext_noop(s)
@@ -66,6 +66,8 @@
 #define MAX_INVALID_EXTM3U_FILES	16
 #define MAX_INVALID_PLS_FILES		16
 #define MAX_INVALID_PLS_LINES		16
+#define EXT_INFO_DEFAULT		"DLNA.ORG_PS=1;DLNA.ORG_CI=0;DLNA.ORG_OP=00;DLNA.ORG_FLAGS=00000000000000000000000000000000"
+#define EXT_INFO_PN			";DLNA.ORG_PN="
 
 typedef enum {
     FILE_MP3,
@@ -74,10 +76,16 @@ typedef enum {
     FILE_RIFF_WAVE,
     FILE_M4A,
     FILE_OGG,
+    FILE_MPG,
+    FILE_MP4,
     FILE_PLS,
     FILE_M3U,
     FILE_EXTM3U,
+    FILE_BMP,
+    FILE_GIF,
     FILE_JPG,
+    FILE_PNG,
+    FILE_TIFF,
     FILE_UNKNOWN,
     FILE_TYPES_COUNT,
 } FileType;
@@ -89,6 +97,34 @@ struct _InodeList {
     ino_t node;
 };
 
+/*struct {
+    char *mime_type;
+    char *names;
+    char *descs;
+    ItemClass item_class;
+    char *dlna_pn;
+} file_types[];*/
+
+static const char *file_type_dlna_pn[] = {
+    [FILE_MP3] 		= "MP3",
+    [FILE_MP3_ID3]	= "MP3",
+    [FILE_WMA] 		= "WMAFULL",
+    [FILE_RIFF_WAVE] 	= NULL,
+    [FILE_M4A] 		= NULL,
+    [FILE_OGG]		= NULL,
+    [FILE_MPG] 		= NULL,
+    [FILE_MP4] 		= NULL,
+    [FILE_PLS] 		= NULL,
+    [FILE_M3U] 		= NULL,
+    [FILE_EXTM3U] 	= NULL,
+    [FILE_BMP]		= NULL,
+    [FILE_GIF]		= NULL,
+    [FILE_JPG]		= "JPEG_TN",
+    [FILE_PNG]		= NULL,
+    [FILE_TIFF]		= NULL,
+    [FILE_UNKNOWN] 	= NULL,
+};
+
 static const char *file_type_mime_types[] = {
     [FILE_MP3] 		= "audio/mpeg",
     [FILE_MP3_ID3]	= "audio/mpeg",
@@ -96,10 +132,16 @@ static const char *file_type_mime_types[] = {
     [FILE_RIFF_WAVE] 	= "audio/x-wav",
     [FILE_M4A] 		= "audio/mp4",
     [FILE_OGG]		= "audio/vorbis",
+    [FILE_MPG] 		= "video/mpeg",
+    [FILE_MP4] 		= "video/mp4",
     [FILE_PLS] 		= "audio/x-scpls",
     [FILE_M3U] 		= "audio/m3u",
     [FILE_EXTM3U] 	= "audio/m3u",
+    [FILE_BMP]		= "image/bmp",
+    [FILE_GIF]		= "image/gif",
     [FILE_JPG]		= "image/jpeg",
+    [FILE_PNG]		= "image/png",
+    [FILE_TIFF]		= "image/tiff",
     [FILE_UNKNOWN] 	= "application/octet-stream",
 };
 
@@ -110,10 +152,16 @@ static const char *file_type_names[] = {
     [FILE_RIFF_WAVE] 	= "wav",
     [FILE_M4A] 		= "m4a",
     [FILE_OGG]		= "ogg",
+    [FILE_MPG] 		= "mpg",
+    [FILE_MP4] 		= "mp4",
     [FILE_PLS] 		= "pls",
     [FILE_M3U] 		= "m3u",
     [FILE_EXTM3U] 	= "m3u", /* possibly extm3u in the future */
+    [FILE_BMP]		= "bmp",
+    [FILE_GIF]		= "gif",
     [FILE_JPG]		= "jpg",
+    [FILE_PNG]		= "png",
+    [FILE_TIFF]		= "tiff",
     [FILE_UNKNOWN] 	= "unknown",
 };
 
@@ -123,12 +171,18 @@ static const char *file_type_descs[] = {
     [FILE_MP3_ID3]	= "MPEG-1 Audio Layer 3 (MP3) with leading ID3 tag",
     [FILE_WMA] 		= "Windows Media Audio (WMA)",
     [FILE_RIFF_WAVE] 	= "RIFF Wave",
-    [FILE_M4A] 		= "MPEG v4 iTunes AAC-LC (M4A)",
+    [FILE_M4A] 		= "MPEG v4 Audio (M4A)",
     [FILE_OGG]		= "Ogg Vorbis audio",
+    [FILE_MPG] 		= "MPEG v1/2 Video (MPG)",
+    [FILE_MP4] 		= "MPEG v4 Video (MP4)",
     [FILE_PLS]		= "PLS playlist",
     [FILE_M3U]		= "Simple M3U playlist",
     [FILE_EXTM3U]	= "Extended M3U playlist",
-    [FILE_JPG]		= "JPEG image"
+    [FILE_BMP]		= "BMP image",
+    [FILE_GIF]		= "GIF image",
+    [FILE_JPG]		= "JPEG image",
+    [FILE_PNG]		= "PNG image",
+    [FILE_TIFF]		= "TIFF image",
 };
 
 static ItemClass file_type_item_classes[] = {
@@ -138,10 +192,16 @@ static ItemClass file_type_item_classes[] = {
     [FILE_RIFF_WAVE]   = ITEM_AUDIO,
     [FILE_M4A]         = ITEM_AUDIO,
     [FILE_OGG]         = ITEM_AUDIO,
+    [FILE_MPG]         = ITEM_VIDEO,
+    [FILE_MP4]         = ITEM_VIDEO,
     [FILE_PLS]         = ITEM_PLAYLIST,
     [FILE_M3U]         = ITEM_PLAYLIST,
     [FILE_EXTM3U]      = ITEM_PLAYLIST,
+    [FILE_BMP]         = ITEM_IMAGE,
+    [FILE_GIF]         = ITEM_IMAGE,
     [FILE_JPG]         = ITEM_IMAGE,
+    [FILE_PNG]         = ITEM_IMAGE,
+    [FILE_TIFF]        = ITEM_IMAGE,
 };
 
 static Entry *scan_entry(const char *fullpath, const char *name, int32_t parent, int indent_size, InodeList *inl);
@@ -274,10 +334,21 @@ check_file_content_type(const char *fullpath)
 	    { FILE_MP3, 	"audio/mpeg" }, /* XXX: FILE_MP3_ID3? */
 	    { FILE_WMA, 	"audio/x-ms-wma" },
 	    { FILE_RIFF_WAVE,	"audio/x-wav" },
-	    { FILE_M4A,		"video/mp4" },
 	    { FILE_M4A,		"audio/mp4" },
 	    { FILE_OGG,		"application/ogg" },
+	    { FILE_MPG,		"video/mpeg" },
+	    { FILE_MPG,		"video/mpv" },
+	    { FILE_MPG,		"video/mp2p" },
+	    { FILE_MPG,		"video/mp2t" },
+	    { FILE_MP4,		"video/mp4" },
+	    { FILE_MP4,		"video/mp4v-es" },
+	    { FILE_MP4,		"video/h264" },
+	    { FILE_MP4,		"video/3gpp" },
+	    { FILE_BMP,		"image/bmp" },
+	    { FILE_GIF,		"image/gif" },
 	    { FILE_JPG,		"image/jpeg" },
+	    { FILE_PNG,		"image/png" },
+	    { FILE_TIFF,	"image/tiff" },
             { 0, },
         };
         for (c = 0; mime_map[c].mime != NULL; c++) {
@@ -324,10 +395,26 @@ check_file_content_type(const char *fullpath)
 }
 
 #ifdef HAVE_ID3LIB
-/* XXX: move this function to a more appropriate place, perhaps lib/iconvme? */
+/* XXX: move this function to a more appropriate place, perhaps lib/striconv? */
 static char *
 iconv_alloc_term(iconv_t *cd, const char *inbuf, size_t inbytes_remaining, size_t nullterm)
 {
+    /* FIXME: use this instead */
+    /*char *outbuf;
+    size_t outbuf_size;
+    int err;
+
+    err = mem_cd_iconv(inbuf, inbytes_remaining, *cd, &outbuf, &outbuf_size);
+    if (err == 0) {
+      if (nullterm != 0) {
+        xrealloc(outbuf, outbuf_size+nullterm);
+        memset(outbuf+outbuf_size, 0, nullterm);
+      }
+    }
+    return err;*/
+
+
+
     char *dest;
     char *outp;
     /* Guess the maximum length the output string can have.  */
@@ -798,6 +885,12 @@ scan_entry(const char *fullpath, const char *name, int32_t parent, int indent_si
 	detail->data.file.item_class = file_type_item_classes[type];
 	/* XXX: dump size, mtime, ?? */
 
+	if (file_type_dlna_pn[type] != NULL) {
+	  detail->data.file.ext_info = xasprintf("%s%s%s", EXT_INFO_DEFAULT, EXT_INFO_PN, file_type_dlna_pn[type]);
+        } else {
+          detail->data.file.ext_info = xstrdup(EXT_INFO_DEFAULT);
+        }
+
 	if (tags_enabled && (type == FILE_MP3 || type == FILE_MP3_ID3)) {
 	    detail = NULL;
 
@@ -1032,6 +1125,7 @@ clear_entries(void)
 		free(d->data.children.list);
 	    } else if (d->type == DETAIL_FILE) {
 	        free(d->data.file.filename);
+                free(d->data.file.ext_info);
 	    } else if (d->type == DETAIL_TAG) {
 		free(d->data.tag.title);
 		free(d->data.tag.artist);
