@@ -1,10 +1,10 @@
 /* upnp.c - Routines dealing interfacing with the upnp library
  *
- * Copyright (C) 2005, 2006  Oskar Liljeblad
+ * Copyright (C) 2005, 2006, 2007  Oskar Liljeblad
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -38,15 +38,10 @@
 #include "gmediaserver.h"
 #include "schemas/MediaServer.h"
 
-/* libupnp 1.2.x has SSDP_PAUSE, but not 1.3.x
- * Perhaps this kludge should be moved somewhere else.
- */
-#ifndef SSDP_PAUSE
-#define SSDP_PAUSE 100
-#endif
+#define GMEDIASERVER_SSDP_PAUSE 100
 
 char *friendly_name = NULL;
-int ssdp_expire_time = SSDP_PAUSE;
+int ssdp_expire_time = GMEDIASERVER_SSDP_PAUSE;
 static UpnpDevice_Handle device;
 static char device_udn[42]; /* uuid:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx + null-byte */
 
@@ -444,7 +439,7 @@ upnp_get_string(ActionEvent *event, const char *key)
 		/* Are we sure empty arguments are reported like this? */
 		return "";
 	    }
-	    return ixmlNode_getNodeValue(node);
+	    return (char *) ixmlNode_getNodeValue(node); /* XXX: const? */
 	}
     }
 
